from pathlib import Path

import joblib
from loguru import logger
from sklearn.neighbors import KNeighborsClassifier

from ezancestry.config import aisnps_set as _aisnps_set
from ezancestry.config import algorithm as _algorithm
from ezancestry.config import models_directory as _models_directory
from ezancestry.config import population_level as _population_level


def train(
    df,
    labels,
    algorithm=None,
    aisnps_set=None,
    k=None,
    models_directory=None,
    population_level=None,
    overwrite_model=False,
):
    """Fit and return a KNeighborsClassifier model (and optionally save it when overwite_model=True)

    :param df: Should be either the df_encoded or df_reduced DataFrmae
    :type df: pandas DataFrame
    :labels: The labels for the df
    :type labels: array, list
    :param algorithm: The dimensionality reduction algorithm that was used
    :type algorithm: str
    :param aisnps_set: The set of AISNPs to use
    :type aisnps_set: str
    :param k: The number of nearest neighbors to consider
    :type k: int
    :models_directory: The directory to save the models to
    :type models_directory: str
    :param population_level: The population level to use
    :type population_level: str
    :param overwrite_model: Whether to overwrite the model if it already exists
    :type overwrite_model: bool
    :return: The trained KNN model
    :rtype: fit KNeighborsClassifier
    """

    if k is None:
        k = _k
    if population_level is None:
        population_level = _population_level
    if aisnps_set is None:
        aisnps_set = _aisnps_set
    if algorithm is None:
        algorithm = _algorithm
    if models_directory is None:
        models_directory = _models_directory

    models_directory = Path(models_directory)

    algorithm = algorithm.upper()
    aisnps_set = aisnps_set.upper()
    population_level = (
        population_level.replace("-", "").replace(" ", "").upper()
    )

    # Create the model
    model = KNeighborsClassifier(n_neighbors=k, weights="distance", n_jobs=4)
    model.fit(df, labels)

    # Save the model
    if overwrite_model:
        joblib.dump(
            model,
            models_directory.joinpath(
                f"KNN.{algorithm}.{aisnps_set}.{population_level}.bin"
            ),
        )
        logger.info(f"Wrote the KNN model to: {models_directory}")

    return model


def predict_ancestry(df, trained_model):
    """Predict the ancestry for a given DataFrame and return it

    :param df: The df_encoded or df_reduced DataFrame to predict on.
    :type df: pandas DataFrame
    :param trained_model: Path to the trained model, or the model itself
    :type trained_model: str or fit KNeighborsClassifier
    :return: A dataframe with the predictions
    :rtype: pandas DataFrame
    """
    ancestrydf = df.copy()
    try:
        knn = joblib.load(str(trained_model))
        logger.info(f"Successfully loaded trained KNN model: {trained_model}")
    except Exception as e:
        logger.error(f"Could not load the KNN model: {e}")
        knn = trained_model
        logger.info("Using user-provided KNN model")

    user_pop = knn.predict(ancestrydf)
    user_pop_probs = knn.predict_proba(ancestrydf)

    ancestrydf["predicted_population"] = user_pop
    ancestrydf[knn.classes_] = user_pop_probs

    return ancestrydf



import ftplib
import sys
from pathlib import Path

from loguru import logger

from ezancestry.config import \
    thousand_genomes_directory as _thousands_genomes_directory


def download_thousand_genomes(thousand_genomes_directory=None):
    """Downloads the 1000 Genomes files to your local machine.

    :param thousand_genomes_directory: Full file path to a directory where you want to download the 1000 Genomes files.
    :type thousand_genomes_directory: str
    """

    if thousand_genomes_directory is None:
        thousand_genomes_directory = _thousands_genomes_directory
    thousand_genomes_directory = Path(thousand_genomes_directory)

    if not Path(thousand_genomes_directory).exists():
        sys.exit("Please enter a valid path: Exiting...")

    empty_directory = not any(Path(thousand_genomes_directory).iterdir())
    if empty_directory:
        if (
            input(
                "Are you sure you want to download 1000 Genomes data (13GB)? (y/n)"
            ).upper()
            != "Y"
        ):
            sys.exit("Exiting...")

    ftp_site = "ftp.1000genomes.ebi.ac.uk"
    filepath = "/vol1/ftp/release/20130502/supporting/bcf_files/"
    ftp = ftplib.FTP(ftp_site)
    ftp.login()
    ftp.cwd(filepath)

    logger.info("Downloading 1000 Genomes data, this will take about an hour")
    for chromosome in [str(_) for _ in range(1, 23)] + ["X", "Y"]:
        # bcf file
        bcf_file = f"ALL.chr{chromosome}.phase3_shapeit2_mvncall_integrated_v5.20130502.genotypes.bcf"
        if chromosome == "X" or chromosome == "Y":
            bcf_file = f"ALL.chr{chromosome}.phase3_shapeit2_mvncall_integrated.20130502.genotypes.bcf"

        # check if the file exists
        if not Path(thousand_genomes_directory).joinpath(bcf_file).exists():
            with open(
                Path(thousand_genomes_directory).joinpath(bcf_file), "wb"
            ) as fp:
                logger.info(f"Downloading chromosome {chromosome}...")
                ftp.retrbinary(f"RETR {bcf_file}", fp.write)
        else:
            logger.warning(
                f"{Path(thousand_genomes_directory).joinpath(bcf_file)} already exists, skipping..."
            )

        # index file
        index_file = f"ALL.chr{chromosome}.phase3_shapeit2_mvncall_integrated_v5.20130502.genotypes.bcf.csi"
        if chromosome == "X" or chromosome == "Y":
            index_file = f"ALL.chr{chromosome}.phase3_shapeit2_mvncall_integrated.20130502.genotypes.bcf.csi"

        # check if the file exists
        if not Path(thousand_genomes_directory).joinpath(index_file).exists():
            with open(
                Path(thousand_genomes_directory).joinpath(index_file), "wb"
            ) as fp:
                ftp.retrbinary(f"RETR {index_file}", fp.write)
        else:
            logger.warning(
                f"{Path(thousand_genomes_directory).joinpath(index_file)} already exists, skipping..."
            )



from pathlib import Path

import joblib
import pandas as pd
import umap
from loguru import logger
from sklearn.decomposition import PCA
from sklearn.neighbors import NeighborhoodComponentsAnalysis as NCA

from ezancestry.config import models_directory as _models_directory


def dimensionality_reduction(
    df,
    algorithm="PCA",
    aisnps_set="Kidd",
    n_components=3,
    overwrite_model=False,
    labels=None,
    population_level="superpopulation",
    models_directory=None,
    random_state=None,
):
    """Reduce the dimensionality of the AISNPs
    :param df: One-hot encoded AISNPs.
    :type df: pandas DataFrame
    :param algorithm: The type of dimensionality reduction to perform.
        One of {PCA, UMAP, t-SNE, NCA}
    :type algorithm: str
    :param aisnps_set: One of either {Kidd, Seldin}
    :type aisnps_set: str
    :param n_components: The number of components to use for dimensionality reduction.
    :param n_components: int
    :param overwrite_model: Whether to overwrite the model.
    :type overwrite_mode: bool
    :param labels: If algorithm is "NCA", then labels for the records in the
    DataFrame are used for the dimensionality reduction.
    :type labels: array-like
    :param population_level: Either {"population" or "super population"} If
    algorithm is "NCA", then population_level is used to name the
    dimensionality reduction model file.
    :type population_level: str
    :param models_directory: The directory to save the model to.
    :type models_directory: str
    :param random_state: The random_state is used to seed the dimensionality reduction model. If None (default), then a random seed is chosen.
    :type random_state: int or None
    :returns: The transformed X DataFrame, reduced to 3 components by <algorithm>.
    """
    algorithm = algorithm.upper()
    aisnps_set = aisnps_set.upper()

    if models_directory is None:
        models_directory = _models_directory
    models_directory = Path(models_directory)

    population_level = (
        population_level.replace("-", "").replace(" ", "").upper()
    )
    columns = [
        col
        for col in df.columns
        if col not in ("gender", "population", "superpopulation")
    ]
    df = df[columns]
    if overwrite_model:
        if algorithm in set(["PCA", "UMAP"]):
            if algorithm == "PCA":
                reducer = PCA(
                    n_components=n_components, random_state=random_state
                )
            elif algorithm == "UMAP":
                reducer = umap.UMAP(
                    n_components=n_components,
                    min_dist=0.2,
                    metric="dice",
                    random_state=random_state,
                )
            else:
                pass
            df_reduced = reducer.fit_transform(df.values)
            joblib.dump(
                reducer,
                models_directory.joinpath(f"{algorithm}.{aisnps_set}.bin"),
            )
        elif algorithm == "NCA":
            reducer = NCA(
                n_components=n_components,
                random_state=random_state,
            )
            df_reduced = reducer.fit_transform(df.values, labels)
            joblib.dump(
                reducer,
                models_directory.joinpath(
                    f"{algorithm}.{aisnps_set}.{population_level}.bin"
                ),
            )
        else:
            return None

        logger.info(
            f"Successfully wrote a dimensionality reduction model to {models_directory}"
        )
    else:
        # load reducer and return reduced X
        if algorithm in set(["PCA", "UMAP"]):
            try:
                reducer = joblib.load(
                    models_directory.joinpath(f"{algorithm}.{aisnps_set}.bin")
                )
            except FileNotFoundError:
                return None
        elif algorithm == "NCA":
            try:
                reducer = joblib.load(
                    models_directory.joinpath(
                        f"{algorithm}.{aisnps_set}.{population_level}.bin"
                    )
                )
            except FileNotFoundError:
                return None
        else:
            logger.warning(
                "Could not find a matching dimensionality reduction model on disk for your algorithm type"
            )
        df_reduced = reducer.transform(df.values)
        logger.info("Successfully loaded a dimensionality reduction model")

    return pd.DataFrame(df_reduced, columns=["x", "y", "z"], index=df.index)



import configparser
import importlib.resources as pkg_resources
import os
import shutil
from pathlib import Path

from loguru import logger

from ezancestry import __project__

# setup the project directory in the user's home diretory
project_directory = Path.home().joinpath(f".{__project__}")
Path(project_directory).mkdir(parents=True, exist_ok=True)

default_data_directory = Path(project_directory).joinpath("data")
default_models_directory = default_data_directory.joinpath("models")
default_aisnps_directory = default_data_directory.joinpath("aisnps")
default_samples_directory = default_data_directory.joinpath("samples")
default_thousand_genomes_directory = default_data_directory.joinpath(
    "thousand_genomes"
)

default_data_directory.mkdir(parents=True, exist_ok=True)
default_models_directory.mkdir(parents=True, exist_ok=True)
default_aisnps_directory.mkdir(parents=True, exist_ok=True)
default_samples_directory.mkdir(parents=True, exist_ok=True)
default_thousand_genomes_directory.mkdir(parents=True, exist_ok=True)

full_config_file_path = Path(project_directory).joinpath("conf.ini")

config = configparser.ConfigParser()

# make sure the non-1kG default directories have data in them
if not any(default_models_directory.iterdir()):
    # pkg_resources copy data from the package to the user's home directory
    logger.info(
        f"Copying models to {default_models_directory}, this only happens once..."
    )
    for fileobj in pkg_resources.contents("data.models"):
        if not str(fileobj).endswith(".bin"):
            continue
        with pkg_resources.path("data.models", fileobj) as file_to_copy:
            shutil.copy(file_to_copy, default_models_directory)


if not any(default_aisnps_directory.iterdir()):
    # pkg_resources copy data from the package to the user's home directory
    logger.info(
        f"Copying aisnps data to {default_aisnps_directory}, this only happens once..."
    )
    for fileobj in pkg_resources.contents("data.aisnps"):
        if fileobj == "__init__.py" or fileobj == "__pycache__":
            continue
        with pkg_resources.path("data.aisnps", fileobj) as file_to_copy:
            shutil.copy(file_to_copy, default_aisnps_directory)

if not any(default_samples_directory.iterdir()):
    # pkg_resources copy data from the package to the user's home directory
    logger.info(
        f"Copying samples data to {default_samples_directory}, this only happens once..."
    )
    for fileobj in pkg_resources.contents("data.samples"):
        if fileobj == "__init__.py" or fileobj == "__pycache__":
            continue
        with pkg_resources.path("data.samples", fileobj) as file_to_copy:
            shutil.copy(file_to_copy, default_samples_directory)

# Set default values
config["directories"] = {
    "data_directory": default_data_directory,
    "models_directory": default_models_directory,
    "aisnps_directory": default_aisnps_directory,
    "samples_directory": default_samples_directory,
    "thousand_genomes_directory": default_thousand_genomes_directory,
}
config["general"] = {
    "population_level": "superpopulation",
    "aisnps_set": "Kidd",
}
config["dimensionality_reduction"] = {
    "algorithm": "PCA",
    "n_components": 3,
}
config["nearest_neighbors"] = {
    "k": 9,
}

if (
    not os.path.exists(full_config_file_path)
    or os.stat(full_config_file_path).st_size == 0
):
    with open(full_config_file_path, "w") as configfile:
        config.write(configfile)

# Read config file
config.read(full_config_file_path)
data_directory = config["directories"]["data_directory"]
models_directory = config["directories"]["models_directory"]
aisnps_directory = config["directories"]["aisnps_directory"]
samples_directory = config["directories"]["samples_directory"]
thousand_genomes_directory = config["directories"]["thousand_genomes_directory"]
population_level = config["general"]["population_level"]
aisnps_set = config["general"]["aisnps_set"]
algorithm = config["dimensionality_reduction"]["algorithm"]
n_components = int(config["dimensionality_reduction"]["n_components"])
k = int(config["nearest_neighbors"]["k"])

if not data_directory:
    data_directory = Path(__file__).parents[1].joinpath("data")
if not models_directory:
    models_directory = data_directory.joinpath("models")
if not aisnps_directory:
    aisnps_directory = data_directory.joinpath("aisnps")
if not samples_directory:
    samples_directory = data_directory.joinpath("samples")
if not thousand_genomes_directory:
    thousand_genomes_directory = data_directory.joinpath("thousand_genomes")



import sys
from pathlib import Path

import pandas as pd
from cyvcf2 import VCF, Writer
from loguru import logger

from ezancestry.config import aisnps_directory as _aisnps_directory
from ezancestry.config import aisnps_set as _aisnps_set
from ezancestry.config import data_directory as _data_directory
from ezancestry.config import \
    thousand_genomes_directory as _thousand_genomes_directory
from ezancestry.fetch import download_thousand_genomes


def extract_aisnps(
    thousand_genomes_directory=None,
    aisnps_file=None,
    aisnps_set=None,
    aisnps_directory=None,
):
    """Extract the AISNP SNPs from the 1000 Genomes data. The thosuand_genomes_directory must be populated with data.

    :param thousand_genomes_directory: Full path to the directory where the 1000 genomes bcf files are located.
    :type thousand_genomes_directory: str
    :param aisnps_file: Full path to the file containing the AISNP SNPs.
    :type aisnps_file: str
    :param aisnps_set: Which set of AISNP SNPs to extract.
    :type aisnps_set: str
    :param aisnps_directory: Full path to the directory where the AISNP SNPs will be written.
    :type aisnps_directory: str
    """

    # default to the config if None
    if thousand_genomes_directory is None:
        thousand_genomes_directory = _thousand_genomes_directory
    thousand_genomes_directory = Path(thousand_genomes_directory)

    if aisnps_directory is None:
        aisnps_directory = _aisnps_directory
    aisnps_directory = Path(aisnps_directory)

    if aisnps_file is None:
        aisnps_file = aisnps_directory.joinpath(f"{aisnps_set}.AISNP.txt")
        logger.info(f"Using: {aisnps_set}.AISNP.txt")
    if aisnps_set is None:
        aisnps_set = _aisnps_set

    # read the AISNP file
    try:
        df = pd.read_csv(aisnps_file, sep="\t", dtype=str)
    except FileNotFoundError:
        logger.error(
            "Please check the path to the AISNPs file (KIDD.AISNP.txt)"
        )
        sys.exit(1)

    logger.info(
        f"Looking for 1000 genomes data in: {thousand_genomes_directory}"
    )
    bcf_fname = f"ALL.chr{1}.phase3_shapeit2_mvncall_integrated_v5.20130502.genotypes.bcf"
    bcf_file = thousand_genomes_directory.joinpath(bcf_fname)
    bcf = VCF(bcf_file)
    outfile = aisnps_directory.joinpath(f"{aisnps_set}.AISNP.1kG.vcf")
    w = Writer(outfile, bcf)
    for _, aim in df.iterrows():
        rsid = aim["rsid"]
        chrom = aim["chromosome"]
        pos = aim["position_hg19"]
        bcf_fname = f"ALL.chr{chrom}.phase3_shapeit2_mvncall_integrated_v5.20130502.genotypes.bcf"
        bcf_file = thousand_genomes_directory.joinpath(bcf_fname)
        bcf = VCF(bcf_file)
        snp_writer = 0
        for variant in bcf("{}:{}-{}".format(chrom, pos, pos)):
            if variant.POS == int(pos):
                w.write_record(variant)
                snp_writer += 1
        if snp_writer == 0:
            logger.warning(
                f"Couldn't find {rsid} on chromosome {chrom} position {pos}"
            )
    w.close()
    bcf.close()
    logger.info(f"Successfully wrote {aisnps_set}.AISNP.1kG.vcf")


def _generate_aisnps(
    thousand_genomes_directory=None, aisnps_set=None, aisnps_directory=None
):
    """A utility function to download the 1000 Genomes data, if necessary. Then create the AISNPs vcf files.

    :param thousand_genomes_directory: Full path to the directory where the 1000 genomes bcf files are located.
    :type thousand_genomes_directory: str
    :param aisnps_set: Which set of AISNP SNPs to extract.
    :type aisnps_set: str
    :param aisnps_directory: Full path to the directory where the AISNP SNPs will be written.
    :type aisnps_directory: str
    """

    # default to the config if None
    if thousand_genomes_directory is None:
        thousand_genomes_directory = _thousand_genomes_directory
    if aisnps_directory is None:
        aisnps_directory = _aisnps_directory
    if aisnps_set is None:
        aisnps_set = _aisnps_set

    download_thousand_genomes(thousand_genomes_directory)
    extract_aisnps(
        thousand_genomes_directory,
        aisnps_directory=aisnps_directory,
        aisnps_set="Kidd",
    )
    extract_aisnps(
        thousand_genomes_directory,
        aisnps_directory=aisnps_directory,
        aisnps_set="Seldin",
    )



import webbrowser
from enum import Enum
from pathlib import Path
from typing import Optional

import pandas as pd
import plotly.express as px
import typer
from loguru import logger
from sklearn.model_selection import train_test_split

from ezancestry.aisnps import extract_aisnps
from ezancestry.config import aisnps_directory as _aisnps_directory
from ezancestry.config import aisnps_set as _aisnps_set
from ezancestry.config import algorithm as _algorithm
from ezancestry.config import k as _k
from ezancestry.config import models_directory as _models_directory
from ezancestry.config import n_components as _n_components
from ezancestry.config import population_level as _population_level
from ezancestry.config import samples_directory as _samples_directory
from ezancestry.config import \
    thousand_genomes_directory as _thousand_genomes_directory
from ezancestry.dimred import dimensionality_reduction
from ezancestry.evaluate import export_performance
from ezancestry.fetch import download_thousand_genomes
from ezancestry.model import predict_ancestry, train
from ezancestry.process import (encode_genotypes, get_1kg_labels,
                                process_user_input, vcf2df)


class PopulationLevel(str, Enum):
    population = "population"
    super_population = "superpopulation"


app = typer.Typer(
    help="The ezancestry command line tool makes it easy to quickly train models "
    "on 1000 genomes project data and view the performance of a "
    "ancestry-informative snp set (AISNPs) at predicting genetic ancestry."
)


@app.command(short_help="Build and evaluate a new model from a set of AISNPs.")
def build_model(
    models_directory: str = typer.Option(
        None, help="The path to the directory to save the model to."
    ),
    aisnps_directory: str = typer.Option(
        None,
        help="The path to the directory where the AISNPs files are located.",
    ),
    n_components: int = typer.Option(
        None,
        help="The number of components to use in the PCA dimensionality reduction.",
    ),
    k: int = typer.Option(
        None, help="The number of nearest neighbors to use in the KNN model."
    ),
    thousand_genomes_directory: str = typer.Option(
        None, help="The path to the 1000 genomes directory."
    ),
    samples_directory: str = typer.Option(
        None, help="The path to the directory containing the samples."
    ),
    population_level: PopulationLevel = typer.Option(
        PopulationLevel.super_population,
        help="The granularity of genetic ancestry you want to predict.",
    ),
    algorithm: str = typer.Option(
        None,
        help="The dimensionality reduction algorithm to use. Use one of pca|umap|nca",
    ),
    aisnps_set: Optional[str] = typer.Option(
        None,
        help="The name of the AISNP set to use. To start, choose either "
        "'Kidd' or 'Seldin'. The default value in conf.ini is 'Kidd'."
        "\n*If using your AISNP set, this value will be the in the naming"
        "convention for all the new model files that are created*",
    ),
):
    """
    For example, if you create a custom AISNP file here: ~/.ezancestry/data/aisnps/custom.AISNP.txt
    and then run the build-model command, you will build a model from the snps in that file:

    $ ezancestry build-model --aisnps-set custom

    See github.com/ezancestry/ezancestry/data/aisnps/custom.AISNP.txt for an example of a custom AISNP file.

    * Note that the 1000 genomes dataset is required for this function to work. *

    * Default arguments are from the ~/.ezancestry/conf.ini file. *
    """

    if models_directory is None:
        models_directory = _models_directory
    if aisnps_directory is None:
        aisnps_directory = _aisnps_directory
    if population_level is None:
        population_level = _population_level
    if algorithm is None:
        algorithm = _algorithm
    if n_components is None:
        n_components = _n_components
    if k is None:
        k = _k
    if thousand_genomes_directory is None:
        thousand_genomes_directory = _thousand_genomes_directory
    if aisnps_set is None:
        aisnps_set = _aisnps_set
    if samples_directory is None:
        samples_directory = _samples_directory

    models_directory = Path(models_directory)
    aisnps_directory = Path(aisnps_directory)
    samples_directory = Path(samples_directory)
    thousand_genomes_directory = Path(thousand_genomes_directory)

    # download 1kG
    download_thousand_genomes(thousand_genomes_directory)
    # extract snps
    aisnps_file = Path(aisnps_directory).joinpath(f"{aisnps_set}.AISNP.txt")
    extract_aisnps(thousand_genomes_directory, aisnps_file, aisnps_set)

    # process data
    dfsamples = get_1kg_labels(samples_directory)
    vcf_fname = Path(aisnps_directory).joinpath(f"{aisnps_set}.AISNP.1kG.vcf")
    dfsnps = vcf2df(vcf_fname, dfsamples)
    labels = dfsnps[population_level]
    dfsnps.drop(
        columns=["population", "superpopulation", "gender"], inplace=True
    )

    # split the training and test data
    train_df, test_df, y_train, y_test = train_test_split(
        dfsnps,
        labels,
        test_size=0.4,
        stratify=labels,
        random_state=42,
    )

    # fit & write models on 1kG
    dfencoded_train = encode_genotypes(
        train_df, aisnps_set=aisnps_set, overwrite_encoder=True
    )
    dfencoded_test = encode_genotypes(
        test_df, aisnps_set=aisnps_set, overwrite_encoder=False
    )

    dfreduced_train = dimensionality_reduction(
        dfencoded_train,
        algorithm=algorithm,
        aisnps_set=aisnps_set,
        overwrite_model=True,
        labels=y_train,
        population_level=population_level,
    )
    dfreduced_test = dimensionality_reduction(
        dfencoded_test,
        algorithm=algorithm,
        aisnps_set=aisnps_set,
        overwrite_model=False,
        labels=y_train,
        population_level=population_level,
    )

    knn_model = train(
        dfreduced_train,
        y_train,
        algorithm=algorithm,
        aisnps_set=aisnps_set,
        k=k,
        population_level=population_level,
        overwrite_model=True,
    )

    cv_report, holdout_report = export_performance(
        dfreduced_train,
        dfreduced_test,
        y_train,
        y_test,
        models_directory=models_directory,
        aisnps_directory=aisnps_directory,
        population_level=population_level,
        algorithm=algorithm,
        aisnps_set=aisnps_set,
    )

    return cv_report, holdout_report


@app.command(
    short_help="Predict your genetic ancestry or predict it for a cohort of samples."
)
def predict(
    input_data: str = typer.Argument(
        ...,
        help="Can be a file path to raw genetic data (23andMe, ancestry.com, .vcf) file, a path to a directory containing several raw genetic files, or a (tab or comma) delimited file with sample ids as rows and snps as columns.",
    ),
    output_directory: str = typer.Option(
        None, help="The directory where to write the prediction results file"
    ),
    write_predictions: bool = typer.Option(
        True,
        help="If True, write the predictions to a file. If False, return the predictions as a dataframe.",
    ),
    models_directory: str = typer.Option(
        None,
        help="The path to the directory where the model files are located.",
    ),
    aisnps_directory: str = typer.Option(
        None,
        help="The path to the directory where the AISNPs files are located.",
    ),
    n_components: int = typer.Option(
        None,
        help="The number of components to use in the PCA dimensionality reduction.",
    ),
    k: int = typer.Option(
        None, help="The number of nearest neighbors to use in the KNN model."
    ),
    thousand_genomes_directory: str = typer.Option(
        None, help="The path to the 1000 genomes directory."
    ),
    samples_directory: str = typer.Option(
        None, help="The path to the directory containing the samples."
    ),
    algorithm: str = typer.Option(
        None,
        help="The dimensionality reduction algorithm to use. Choose pca|umap|nca",
    ),
    aisnps_set: Optional[str] = typer.Option(
        None,
        help="The name of the AISNP set to use. To start, choose either "
        "'Kidd' or 'Seldin'. The default value in conf.ini is 'Kidd'."
        "\n*If using your AISNP set, this value will be the in the naming "
        "convention for all the new model files that are created*",
    ),
):
    """
    Predict ancestry from genetic data.

    * Default arguments are from the ~/.ezancestry/conf.ini file. *
    """

    if models_directory is None:
        models_directory = _models_directory
    if aisnps_directory is None:
        aisnps_directory = _aisnps_directory
    if algorithm is None:
        algorithm = _algorithm
    if n_components is None:
        n_components = _n_components
    if k is None:
        k = _k
    if thousand_genomes_directory is None:
        thousand_genomes_directory = _thousand_genomes_directory
    if aisnps_set is None:
        aisnps_set = _aisnps_set
    if samples_directory is None:
        samples_directory = _samples_directory
    if output_directory is None:
        output_directory = Path.cwd()

    output_directory = Path(output_directory)
    models_directory = Path(models_directory)
    aisnps_directory = Path(aisnps_directory)
    samples_directory = Path(samples_directory)
    thousand_genomes_directory = Path(thousand_genomes_directory)

    snpsdf = process_user_input(input_data, aisnps_directory, aisnps_set)
    index = snpsdf.index
    snpsdf = encode_genotypes(
        snpsdf,
        aisnps_set=aisnps_set,
        models_directory=models_directory,
        aisnps_directory=aisnps_directory,
        overwrite_encoder=False,
    )
    pop_predictions = pd.DataFrame()
    superpop_predictions = pd.DataFrame()
    for pop_level in PopulationLevel:
        pop_level = pop_level
        dimreddf = dimensionality_reduction(
            snpsdf,
            algorithm=algorithm,
            aisnps_set=aisnps_set,
            population_level=pop_level,
            models_directory=models_directory,
            overwrite_model=False,
        )
        knn = models_directory.joinpath(
            f"knn.{algorithm}.{aisnps_set}.{pop_level}.bin"
        )
        pop_level_predictions = predict_ancestry(dimreddf, knn)
        if pop_level == "population":
            pop_predictions = pop_predictions.append(pop_level_predictions)
        else:
            superpop_predictions = superpop_predictions.append(
                pop_level_predictions
            )
    predictions = pop_predictions.merge(
        superpop_predictions.drop(columns=["x", "y", "z"]),
        left_index=True,
        right_index=True,
        suffixes=("_population", "_superpopulation"),
    )

    predictions.rename(
        columns={"x": "component1", "y": "component2", "z": "component3"},
        inplace=True,
    )
    predictions.set_index(index, inplace=True)

    if write_predictions:
        predictions.to_csv(output_directory.joinpath("predictions.csv"))
        # store the settings in a commented header
        line = (
            f"#{algorithm},{aisnps_set},{models_directory},{aisnps_directory}"
        )
        with open(output_directory.joinpath("predictions.csv"), "r+") as f:
            content = f.read()
            f.seek(0, 0)
            f.write(line.rstrip("\r\n") + "\n" + content)

        logger.info(
            f"Predictions written to {output_directory}/predictions.csv"
        )

    return predictions


@app.command(
    short_help="Plot the output of the `predict` command against the 1000 genomes samples in 3D"
)
def plot(
    predictions_file: str = typer.Argument(
        ...,
        help="The predictions.csv file from the output of ezancestry predict command (or using predict when using ezancestry as a library).",
    ),
    population_level: PopulationLevel = typer.Option(
        None,
        help="The population level to plot. Can be either 'population' or 'superpopulation'.",
    ),
    output_directory: str = typer.Option(
        None, help="The directory where to write the html plot file"
    ),
):
    """Reads the output of the ezancestry predict command"""

    if output_directory is None:
        output_directory = Path.cwd()
    output_directory = Path(output_directory)
    if population_level is None:
        population_level = _population_level

    with open(predictions_file, "r") as f:
        for line in f:
            if line.startswith("#"):
                (algorithm, aisnps_set, models_directory, aisnps_directory,) = (
                    line.strip("#").strip("\n").split(",")
                )

    predictions = pd.read_csv(predictions_file, index_col=0, comment="#")
    columns = [
        "component1",
        "component2",
        "component3",
        "predicted_population_population",
        "predicted_population_superpopulation",
    ]
    predictions = predictions[columns]
    aisnps_file = Path(aisnps_directory).joinpath(
        f"thousand_genomes.{aisnps_set}.dataframe.csv"
    )
    # don't save the results of these predictions
    aisnps_results = predict(
        aisnps_file,
        output_directory=None,
        models_directory=models_directory,
        aisnps_directory=aisnps_directory,
        thousand_genomes_directory=None,
        samples_directory=None,
        algorithm=algorithm,
        aisnps_set=aisnps_set,
        write_predictions=False,
    )
    aisnps_results = aisnps_results[columns]

    # add a label column to each dataframe that indicates it was user uploaded or not
    aisnps_results["label"] = "thousand_genomes_sample"
    predictions["label"] = "user_uploaded_sample"

    # merge the two dataframes
    df = pd.concat([predictions, aisnps_results])
    df.reset_index(inplace=True)
    df.rename(columns={"index": "sample"}, inplace=True)

    # plot the data
    color_discrete_map = {"user_uploaded_sample": "rgb(0,0,0)"}
    df["size"] = 16
    df.loc[df["label"] == "user_uploaded_sample", "size"] = 150

    if population_level == "superpopulation":
        population_level_column = "predicted_population_superpopulation"
    else:
        population_level_column = "predicted_population_population"

    fig = px.scatter_3d(
        df,
        x="component1",
        y="component2",
        z="component3",
        hover_name="sample",
        color=population_level_column,
        color_discrete_map=color_discrete_map,
        symbol=population_level_column,
        height=800,
        size="size",
        opacity=1.0,
        color_discrete_sequence=[
            "#008fd5",
            "#fc4f30",
            "#e5ae38",
            "#6d904f",
            "#810f7c",
        ],
    )
    fig.update_layout(
        legend=dict(
            orientation="h",
            yanchor="bottom",
            y=1.02,
            xanchor="right",
            x=1,
            font=dict(size=18),
        )
    )

    fig.write_html(output_directory.joinpath("plot.html"))

    webbrowser.open(f"file://{output_directory.joinpath('plot.html')}")

    return fig


@app.command(
    short_help="Generate the data and populate the models in the data/* directories"
)
def generate_dependencies(
    models_directory: str = typer.Option(
        None,
        help="The path to the directory where the model files are located.",
    ),
    aisnps_directory: str = typer.Option(
        None,
        help="The path to the directory where the AISNPs files are located.",
    ),
    thousand_genomes_directory: str = typer.Option(
        None, help="The path to the 1000 genomes directory."
    ),
    samples_directory: str = typer.Option(
        None, help="The path to the directory containing the samples."
    ),
):
    """
    Generate the data and populate the models in the data/* directories.
    """
    if models_directory is None:
        models_directory = _models_directory
    if aisnps_directory is None:
        aisnps_directory = _aisnps_directory
    if thousand_genomes_directory is None:
        thousand_genomes_directory = _thousand_genomes_directory
    if samples_directory is None:
        samples_directory = _samples_directory

    models_directory = Path(models_directory)
    aisnps_directory = Path(aisnps_directory)
    samples_directory = Path(samples_directory)
    thousand_genomes_directory = Path(thousand_genomes_directory)

    k = _k
    n_components = _n_components
    algorithms = ["PCA", "UMAP", "NCA"]
    population_levels = ["population", "superpopulation"]
    aisnps_sets = ["Kidd", "Seldin"]

    for algorithm in algorithms:
        for population_level in population_levels:
            for aisnps_set in aisnps_sets:
                _, _ = build_model(
                    models_directory,
                    aisnps_directory,
                    n_components,
                    k,
                    thousand_genomes_directory,
                    samples_directory,
                    population_level,
                    algorithm,
                    aisnps_set,
                )



import warnings
from pathlib import Path

import joblib
import pandas as pd
from cyvcf2 import VCF
from loguru import logger
from sklearn.preprocessing import OneHotEncoder
from snps import SNPs

from ezancestry.config import aisnps_directory as _aisnps_directory
from ezancestry.config import aisnps_set as _aisnps_set
from ezancestry.config import models_directory as _models_directory
from ezancestry.config import samples_directory as _samples_directory

warnings.simplefilter(action="ignore", category=pd.errors.DtypeWarning)


def get_1kg_labels(samples_directory=None):
    """
    Get the ancestry labels for the 1000 Genomes Project samples.

    :param aisnps_directory: [description]
    :type aisnps_directory: [type]
    :return: DataFrame of sample-level population information
    :rtype: pandas DataFrame
    """
    if samples_directory is None:
        samples_directory = _samples_directory

    dfsamples = pd.read_csv(
        Path(samples_directory).joinpath(
            "integrated_call_samples_v3.20130502.ALL.panel"
        ),
        sep="\t",
    )
    dfsamples.set_index("sample", inplace=True)
    dfsamples.drop(columns=["Unnamed: 4", "Unnamed: 5"], inplace=True)
    dfsamples.columns = ["population", "superpopulation", "gender"]
    return dfsamples


def vcf2df(vcf_fname, dfsamples):
    """Convert a vcf file (from the 1kg AISNPs) to a pandas DataFrame
    :param vcf_fname: path to the vcf file with AISNPs for every 1kg sample
    :type vcf_fname: str
    :param dfsamples: DataFrame with sample-level info on each 1kg sample.
    :type dfsamples: pandas DataFrame
    :return: DataFrame with genotypes for AISNPs as columns and samples as rows.
    :rtype: pandas DataFrame
    """
    vcf_file = VCF(vcf_fname)
    df = pd.DataFrame(index=vcf_file.samples)
    for variant in vcf_file():
        # TODO: ensure un-phasing variants is the desired behavior
        # sorted() normalizes the order of the genotypes
        df[variant.ID] = [
            "".join(sorted(gt.replace("|", ""))) for gt in variant.gt_bases
        ]

    df = df.join(dfsamples, how="inner")

    return df


def encode_genotypes(
    df,
    aisnps_set="Kidd",
    overwrite_encoder=False,
    models_directory=None,
    aisnps_directory=None,
):
    """One-hot encode the genotypes
    :param df: A DataFrame of samples with genotypes as columns
    :type df: pandas DataFrame
    :param aisnps_set: One of either {Kidd, Seldin}
    :type aisnps_set: str
    :param overwrite_encoder: Flag whether or not to overwrite the saved encoder for the given aisnps_set. Default: False, will load the saved encoder model.
    :type overwrite_encoder: bool
    :param models_directory: Path to the directory where the saved encoder model is saved. Default: None, will use the default location.
    :type models_directory: str
    :param aisnps_directory: Path to the directory where the AISNPs are saved. Default: None, will use the default location.
    :type aisnps_directory: str
    :return: pandas DataFrame of one-hot encoded columns for genotypes and OHE instance
    :rtype: pandas DataFrame, OneHotEncoder instance
    """

    if models_directory is None:
        models_directory = _models_directory
    if aisnps_directory is None:
        aisnps_directory = _aisnps_directory

    models_directory = Path(models_directory)
    aisnps_directory = Path(aisnps_directory)

    aisnps_set = aisnps_set.upper()
    try:
        aisnps = pd.read_csv(
            aisnps_directory.joinpath(
                f"thousand_genomes.{aisnps_set}.dataframe.csv"
            ),
            nrows=0,
            index_col=0,
        ).drop(columns=["population", "superpopulation", "gender"])
    except FileNotFoundError:
        logger.critical("""aisnps_set must be either "Kidd" or "Seldin".""")
        return

    # concact will add snps (columns) to the df that aren't in the user-submitted
    # df. Then drop the snps (columns) that are in the user-submitted df, but not
    # in the aisnps set.
    df = pd.concat([aisnps, df])[aisnps.columns]

    # TODO: Impute missing values
    # imputer = KNNImputer(n_neighbors=9)
    # imputed_aisnps = imputer.fit_transform(df)

    if overwrite_encoder:
        # 1. Use a different encoder technique (OHE works for now)
        # 2. Pass a list of valid genotypes (overkill, dimensionality explodes)
        ohe = OneHotEncoder(
            sparse=False,
            handle_unknown="ignore",
        )
        X = ohe.fit_transform(df.values)
        # overwrite the old encoder with a new one
        joblib.dump(
            ohe, models_directory.joinpath(f"one_hot_encoder.{aisnps_set}.bin")
        )
        logger.info(
            f"Wrote a new encoder to {models_directory}/one_hot_encoder.{aisnps_set}.bin"
        )
    else:
        ohe = joblib.load(
            models_directory.joinpath(f"one_hot_encoder.{aisnps_set}.bin")
        )
        logger.info(
            f"Successfully loaded an encoder from {models_directory}/one_hot_encoder.{aisnps_set}.bin"
        )
        X = ohe.transform(df.values)

    return pd.DataFrame(
        X, index=df.index, columns=ohe.get_feature_names(df.columns.tolist())
    )


def process_user_input(input_data, aisnps_directory=None, aisnps_set=None):
    """Process the user-submitted input data.

    :param input_data: [description]
    :type input_data: [type]
    :param aisnps_directory: [description], defaults to None
    :type aisnps_directory: [type], optional
    :param aisnps_set: [description], defaults to None
    :type aisnps_set: [type], optional
    :return: DataFrame where samples are row and genotypes are columns
    :rtype: pandas DataFrame
    """
    if aisnps_directory is None:
        aisnps_directory = _aisnps_directory
    if aisnps_set is None:
        aisnps_set = _aisnps_set

    aisnps_directory = Path(aisnps_directory)

    aisnpsdf = pd.read_csv(
        aisnps_directory.joinpath(f"{aisnps_set}.AISNP.txt"),
        dtype={"rsid": str, "chromosome": str, "position_hg19": int},
        sep="\t",
    )
    
    try:
        input_data_is_pathlike = bool(Path(input_data))
    except TypeError:
        input_data_is_pathlike = False

    # If the user-submitted input ata is a directory, loop over all the files
    # to create a DataFrame of all the input data.
    if input_data_is_pathlike:
        if Path(input_data).is_dir():
            snpsdf = pd.DataFrame(
                columns=[
                    col
                    for col in aisnpsdf.columns
                    if col not in ["rsid", "chromosome", "position_hg19"]
                ]
            )
            for filepath in Path(input_data).iterdir():
                try:
                    snpsdf = pd.concat(
                        [snpsdf, _input_to_dataframe(filepath, aisnpsdf)]
                    )
                except Exception as e:
                    logger.debug(e)
                    logger.warning(f"Skipping {filepath} because it was not valid")
        
            return snpsdf

        # The user-submitted input data is a single file.
        else:
            # _input_to_dataframe needs a Path object
            input_data = Path(input_data)
            try:
                snpsdf = _input_to_dataframe(input_data, aisnpsdf)
                # SNPs will try to read the DataFrame file
                if snpsdf is not None:
                    return snpsdf
                logger.debug(
                    "input_data is not a valid SNPs format, that's ok, trying to read as a pre-formatted DataFrame"
                )
            except Exception as e:
                logger.debug(e)
            # read the user-submitted preformatted data as a DataFrame
            try:
                snpsdf = pd.read_csv(
                    input_data, index_col=0, sep=None, engine="python", dtype=str
                )
                # Need to clean up the dataframe if there is extra stuff in it
                # keep the first column, it's the index
                cols_to_keep = [snpsdf.columns[0]]
                for col in snpsdf.columns[1:]:
                    if col.startswith("rs"):
                        cols_to_keep.append(col)
                return snpsdf[cols_to_keep]
            except:
                raise ValueError(
                    f"{input_data} is not a valid file or directory. Please provide a valid file or directory."
                )
    else:
        snpsdf = _input_to_dataframe(input_data, aisnpsdf)
    
    return snpsdf


def _input_to_dataframe(input_data, aisnpsdf):
    """Reads one file and returns a pandas DataFrame.

    :param aisnpsdf: A DataFrame of AISNPs
    :type aisnpsdf: pandas DataFrame
    :param input_data: Path object to the file to be read or a SNPs DataFrame
    :type input_data: Path
    :return: A DataFrame of one record and many columns for each AISNP.
    :rtype: pandas DataFrame
    """
    # try to read a single file
    try:
        is_pathlike = bool(Path(input_data))
    except TypeError:
        is_pathlike = False
    if is_pathlike:     
        try:
            snpsobj = SNPs(str(input_data))
            if snpsobj.count == 0:
                logger.debug(f"No snps found in the input_data")
                return None
            snpsdf = snpsobj.snps
        except FileNotFoundError:
            logger.critical(f"Could not find file {input_data}")
        sample_id = Path(input_data).name

    else:
        snpsdf = input_data
        sample_id = "sample"

    snpsdf = snpsdf.reset_index()
    snpsdf.rename(
        columns={"chrom": "chromosome", "pos": "position_hg19"},
        inplace=True,
    )
    # subset to AISNPs
    snpsdf = aisnpsdf.merge(
        snpsdf, on=["rsid", "chromosome", "position_hg19"], how="left"
    )
    # inform user how many missing snps
    n_aisnps = snpsdf["genotype"].notnull().sum()
    n_aisnps_total = snpsdf.shape[0]
    logger.info(
        f"{sample_id} has a valid genotype for {n_aisnps} out of a possible {n_aisnps_total} ({(n_aisnps / n_aisnps_total) * 100}%)"
    )

    snpsdfT = pd.DataFrame(columns=snpsdf["rsid"].tolist())
    snpsdfT.loc[sample_id] = snpsdf["genotype"].tolist()

    return snpsdfT



from pathlib import Path
from typing import Union

import joblib
import pandas as pd
import typer
from loguru import logger
from sklearn.metrics import accuracy_score, classification_report
from sklearn.model_selection import StratifiedKFold, cross_validate

from ezancestry.aisnps import extract_aisnps
from ezancestry.config import aisnps_directory as _aisnps_directory
from ezancestry.config import aisnps_set as _aisnps_set
from ezancestry.config import algorithm as _algorithm
from ezancestry.config import k as _k
from ezancestry.config import models_directory as _models_directory
from ezancestry.config import n_components as _n_components
from ezancestry.config import population_level as _population_level
from ezancestry.config import samples_directory as _samples_directory
from ezancestry.config import \
    thousand_genomes_directory as _thousand_genomes_directory
from ezancestry.dimred import dimensionality_reduction
from ezancestry.fetch import download_thousand_genomes
from ezancestry.model import predict_ancestry, train
from ezancestry.process import encode_genotypes, get_1kg_labels, vcf2df


def export_performance(
    df_train: pd.DataFrame,
    df_test: pd.DataFrame,
    y_train: Union[pd.Series, pd.DataFrame],
    y_test: Union[pd.Series, pd.DataFrame],
    models_directory: str = None,
    aisnps_directory: str = None,
    population_level: str = None,
    algorithm: str = None,
    aisnps_set: str = None,
    outdir: str = None,
):
    """Evaluate the performance of the pre-trained model. For this function to
     work as intended, the models must have already been fit on df_train.

    :param df_train: DataFrame of reduced dimensionality training data.
    :type df_train: pd.DataFrame
    :param df_test: DataFrame of reduced dimensionality test data.
    :type df_test: pd.DataFrame
    :param models_directory: [description], defaults to None
    :type models_directory: str, optional
    :param aisnps_directory: [description], defaults to None
    :type aisnps_directory: str, optional
    :param population_level: [description], defaults to None
    :type population_level: str, optional
    :param algorithm: [description], defaults to None
    :type algorithm: str, optional
    :param n_components: [description], defaults to None
    :type n_components: int, optional
    :param k: [description], defaults to None
    :type k: int, optional
    :param thousand_genomes_directory: [description], defaults to None
    :type thousand_genomes_directory: str, optional
    :param samples_directory: [description], defaults to None
    :type samples_directory: str, optional
    :param aisnps_set: [description], defaults to None
    :type aisnps_set: str, optional
    :param outdir: Where to write the result files, defaults to models_directory
    :type outdir: str, optional
    """

    if models_directory is None:
        models_directory = _models_directory
    if aisnps_directory is None:
        aisnps_directory = _aisnps_directory
    if population_level is None:
        population_level = _population_level
    if algorithm is None:
        algorithm = _algorithm
    if aisnps_set is None:
        aisnps_set = _aisnps_set
    if outdir is None:
        outdir = models_directory

    models_directory = Path(models_directory)
    aisnps_directory = Path(aisnps_directory)
    outdir = Path(outdir)

    population_level = (
        population_level.replace("-", "").replace(" ", "").upper()
    )
    algorithm = algorithm.upper()
    aisnps_set = aisnps_set.upper()

    # Load the pre-trained model
    model = models_directory.joinpath(
        f"knn.{algorithm}.{aisnps_set}.{population_level}.bin"
    )
    model = joblib.load(model)
    logger.info("Predicting ancestry for evaluation...")
    dftrain_results = predict_ancestry(df_train, model)
    dftest_results = predict_ancestry(df_test, model)

    # Perform cross validation
    logger.info("Performing cross validation...")
    cv = StratifiedKFold(n_splits=5, shuffle=True, random_state=42)
    cv_results = cross_validate(
        model,
        df_train,
        y_train,
        cv=cv,
        scoring=["f1_macro", "precision_macro", "recall_macro", "accuracy"],
        return_train_score=True,
        return_estimator=True,
    )
    # Save the CV results
    cv_results_file = outdir.joinpath(
        f"knn.{algorithm}.{aisnps_set}.{population_level}.cv.csv"
    )
    logger.info(f"Writing cross validation results to {cv_results_file}...")
    cv_results = pd.DataFrame(cv_results)
    cv_results.index = [f"Fold {i}" for i in range(1, 6)]
    cv_results["training_accuracy"] = accuracy_score(
        y_train, model.predict(df_train)
    )
    cv_results.to_csv(cv_results_file, index=True)

    # Evaluate on the held out test set
    logger.info("Evaluating on the held out test set...")
    y_pred = dftest_results["predicted_population"]

    holdout_results = pd.DataFrame(
        classification_report(
            y_test, y_pred, target_names=model.classes_, output_dict=True
        )
    )
    holdout_results_file = outdir.joinpath(
        f"knn.{algorithm}.{aisnps_set}.{population_level}.holdout.csv"
    )
    logger.info(
        f"Writing holdout validation results to {holdout_results_file}..."
    )
    holdout_results.to_csv(holdout_results_file, index=True)

    return cv_results, holdout_results



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['data', 'data.aisnps', 'data.models', 'data.samples', 'ezancestry']

package_data = \
{'': ['*']}

install_requires = \
['cmake>=3.21.2,<4.0.0',
 'cyvcf2>=0.30.11,<0.31.0',
 'loguru>=0.5.3,<0.6.0',
 'numpy>=1.21.2,<2.0.0',
 'pandas>=1.3.2,<2.0.0',
 'plotly>=5.3.1,<6.0.0',
 'scikit-learn>=0.24.2,<0.25.0',
 'snps>=2.3.0,<3.0.0',
 'streamlit>=0.87.0,<0.88.0',
 'typer>=0.4.0,<0.5.0',
 'umap-learn>=0.5.1,<0.6.0']

entry_points = \
{'console_scripts': ['ezancestry = ezancestry.commands:app']}

setup_kwargs = {
    'name': 'ezancestry',
    'version': '0.0.3',
    'description': 'Easily predict and visualize genetic ancestry. Evaluate custom ancestry-informative SNP sets.',
    'long_description': '# ezancestry\n\nEasily visualize your direct-to-consumer genetics next to 2500+ samples from the 1000 genomes project. Evaluate the performance of a custom set of ancestry-informative snps (AISNPs) at classifying the genetic ancestry of the 1000 genomes samples using a machine learning model. \n\nA subset of 1000 Genomes Project samples\' single nucleotide polymorphism(s), or, SNP(s) have been parsed from the [publicly available `.bcf` files](ftp.1000genomes.ebi.ac.uk/vol1/ftp/release/20130502/supporting/bcf_files/).  \nThe subset of `SNPs`, AISNPs (ancestry-informative snps), were chosen from two publications:\n  * Set of 55 AISNPs. [Progress toward an efficient panel of SNPs for ancestry inference](https://www.ncbi.nlm.nih.gov/pubmed?db=pubmed&cmd=Retrieve&dopt=citation&list_uids=24508742). Kidd et al. 2014\n  * Set of 128 AISNPs. [Ancestry informative marker sets for determining continental origin and admixture proportions in common populations in America.](https://www.ncbi.nlm.nih.gov/pubmed?cmd=Retrieve&dopt=citation&list_uids=18683858). Kosoy et al. 2009 (Seldin Lab)\n\nezancestry ships with pretrained k-nearest neighbor models for all combinations of following:\n\n    * Kidd (55 AISNPs)\n    * Seldin (128 AISNPs)\n    \n    * continental-level population (superpopulation)\n    * regional population (population)\n \n    * principal componentanalysis (PCA)\n    * neighborhood component analysis (NCA)\n    * uniform manifold approximation and projection (UMAP)\n\n![image](images/ezancestry.gif)\n\n## Table of Contents\n\n* [Installation](#installation)\n* [Config](#config)\n* [Usage](#usage)\n    * [command line tool](#command-line-interface)\n        * [predict](#predict)\n        * [plot](#plot)\n        * [generate-dependencies](#generate-dependencies)\n        * [build-model](#build-model)\n    * [Python API](#python-api)\n* [Visualization](#visualization)\n* [Contributing](#contributing)\n\n\n## Installation\n\nInstall ezancestry with pip:\n\n```shell\npip install ezancestry\n```\n\nOr clone the repository and run `pip install` from the directory:\n\n```shell\ngit clone git@github.com:arvkevi/ezancestry.git\ncd ezancestry\npip install .\n```\n\n## Config\n\nThe first time `ezancestry` is run it will generate a `config.ini` file and `data/` directory in your home directory under `${HOME}/.ezancestry`.\nYou can edit `conf.ini` to change the default settings, but it is not necessary to use ezancestry. The settings are just a utility for the user so they don\'t have to be verbose when interacting with the software. The settings are also keyword arguments to each of the commands in the ezancestry API, so you can always override the default settings.  \n\nThese will be created in your home directory:\n\n```shell\n${HOME}/.ezancestry/conf.ini\n${HOME}/.ezancestry/data/\n```\n\nExplanations of each setting is described in the Options section of the `--help` of each command, for example:\n\n```shell\nezancestry predict --help\n\nUsage: ezancestry predict [OPTIONS] INPUT_DATA\n\n  Predict ancestry from genetic data.\n\n  * Default arguments are from the ~/.ezancestry/conf.ini file. *\n\nArguments:\n  INPUT_DATA  Can be a file path to raw genetic data (23andMe, ancestry.com,\n              .vcf) file, a path to a directory containing several raw genetic\n              files, or a (tab or comma) delimited file with sample ids as\n              rows and snps as columns.  [required]\n\n\nOptions:\n  --output-directory TEXT         The directory where to write the prediction\n                                  results file\n\n  --write-predictions / --no-write-predictions\n                                  If True, write the predictions to a file. If\n                                  False, return the predictions as a\n                                  dataframe.  [default: True]\n\n  --models-directory TEXT         The path to the directory where the model\n                                  files are located.\n\n  --aisnps-directory TEXT         The path to the directory where the AISNPs\n                                  files are located.\n\n  --n-components INTEGER          The number of components to use in the PCA\n                                  dimensionality reduction.\n\n  --k INTEGER                     The number of nearest neighbors to use in\n                                  the KNN model.\n\n  --thousand-genomes-directory TEXT\n                                  The path to the 1000 genomes directory.\n  --samples-directory TEXT        The path to the directory containing the\n                                  samples.\n\n  --algorithm TEXT                The dimensionality reduction algorithm to\n                                  use. Choose pca|umap|nca\n\n  --aisnps-set TEXT               The name of the AISNP set to use. To start,\n                                  choose either \'Kidd\' or \'Seldin\'. The\n                                  default value in conf.ini is \'Kidd\'. *If\n                                  using your AISNP set, this value will be the\n                                  in the namingc onvention for all the new\n                                  model files that are created*\n\n  --help                          Show this message and exit.\n```\n\n## Usage\n\nezancestry can be used as a command-line tool or as a Python library. ezancestry `predict` comes with pre-trained models when `--aisnps-set="Kidd"` (default) or `--aisnps-set="Seldin"`.\n\n`build-model` and `generate-dependencies` are for advanced users -- they download large amounts of data and build a new model from a custom AISNPs file.\n\n### command-line interface\n\nThere are four commands available:\n\n1. `predict`: predict the genetic ancestry of a sample or cohort of samples using the nearest neighbors model.\n2. `plot`: plot the genetic ancestry of samples using only the output of `predict`.\n3. `generate-dependencies`: generate the dependencies for `build-model`.\n4. `build-model`: build a nearest neighbors model from the 1000 genomes data using a custom set of AISNPs. Requires: `generate-dependencies` to be run first.\n\nUse the commands in the following way:\n\n\n#### predict\n\nezancestry can predict the genetic ancestry of a sample or cohort of samples using the nearest neighbors model.\nThe `input_data` can be a file path to raw genetic data (23andMe, ancestry.com, .vcf) file, a path to a directory containing several raw genetic files, or a (tab or comma) delimited file with sample ids as rows and snps as columns.\n\nThis writes a file, `predictions.csv` to the `output_directory` (defaults to current directory). This file contains predicted ancestry for each sample.\n\n**Direct-to-consumer genetic data file (23andMe, ancestry.com, etc.)**:\n\n```shell\nezancestry predict mygenome.txt\n```\n\n**Directory of direct-to-consumer genetic data files or .vcf files**:\n\n```shell\nezancestry predict /path/to/genetic_datafiles\n```\n\n**comma-separated file with sample ids as rows and snps as columns, filled with genotypes as values**\n\n```shell\nezancestry predict ${HOME}/.ezancestry/data/aisnps/thousand_genomes.KIDD.dataframe.csv\n```\n\n#### plot\n\nVisualize the output of `predict` using the `plot` command. This will open a 3d scatter plot in a browser.\n\n```shell\nezancestry plot predictions.csv\n```\n\n#### generate-dependencies\n\nThis command will download all of the data required to build a new nearest neighbors model for a custom set of AISNPs. \n**This command will attempt to download all the .bcf files from The 1000 Genomes Project.** If you want to use existing models, see `predict` and `plot`.\n\nWithout any arguments this command will download all necessary data to build new models and put it in the `${HOME}/.ezancestry/data/` directory.\n\n```shell\nezancestry generate-dependencies\n```\n\nNow you are ready to build a new model with `build-model`.\n\n#### build-model\n\nTest the discriminative power of your custom set of AISNPs.\n\nThis command will build all the necessary models to visualize and predict the 1000 genomes samples as well as user-uploaded samples. A model performace evaluation report will be generated for a five-fold cross-validation on the training set of the 1000 genomes samples as well as a report for the holdout set.\n\nCreate a custom AISNP file here: `~/.ezancestry/data/aisnps/custom.AISNP.txt`. The prefix of the filename, `custom`, can be whatever you want. Note that this value is used as the `aisnps-set` keyword argument for other ezancestry commands.\n\nThe file should look like this:\n```\nid      chromosome      position_hg19\nrs731257        7       12669251\nrs2946788       11      24010530\nrs3793451       9       71659280\nrs10236187      7       139447377\nrs1569175       2       201021954\n```\n\n```shell\nezancestry build-model --aisnps-set=custom\n```\n\n### Python API\n\nSee the [notebook](github.com/arvkevi/ezancestry/ezancestry_library_tutorial.ipynb)\n\n### Visualization\n\n[http://ezancestry.herokuapp.com/](http://ezancestry.herokuapp.com/) \n\n[Open in Streamlit](https://share.streamlit.io/arvkevi/ezancestry/streamlit-share/streamlit/app.py)\n\n![image](images/ezancestry.png)\n\n### Contributing\n\nContributions are welcome! Please feel free to create an issue for discussion or make a pull request.\n',
    'author': 'arvkevi',
    'author_email': 'arvkevi@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)
